/*****************************************************************************
 * Copyright (C) 2008  Eus (software_freedom at member dot fsf dot org)      *
 *                                                                           *
 * This program is free software: you can redistribute it and/or modify      *
 * it under the terms of the GNU General Public License as published by      *
 * the Free Software Foundation, either version 3 of the License, or         *
 * (at your option) any later version.                                       *
 *                                                                           *
 * This program is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of            *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
 * GNU General Public License for more details.                              *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.     *
 *****************************************************************************/

package phoneNumberMnemonics;

/**
 * Wheel is a ring-shaped object with markings on its surface. The wheel can be
 * turned only in one direction. One mark can be read in each turn.
 *
 * @author Eus
 */
public class Wheel implements Cloneable
{
	private String [] markings;
	private int nthMark;

	/**
	 * Constructs a Wheel object with certain markings.
	 *
	 * @param markings the markings to be put on the wheel's surface
	 *
	 * @throws IllegalArgumentException if markings is null or empty
	 */
	public Wheel (String [] markings)
	{
		if (markings == null || markings.length == 0)
		{
			throw new IllegalArgumentException (
				"markings is null or empty"
			);
		}

		this.markings = markings;
		nthMark = 0;
	}

	/**
	 * Turns the wheel to show the next marking.
	 *
	 * @return true if this wheel undergoes a roll over. Otherwise, false.
	 */
	public boolean turn ()
	{
		boolean rc = nthMark == markings.length - 1;

		nthMark = (nthMark + 1) % markings.length;
		return rc;
	}

	/**
	 * Reads the current marking.
	 *
	 * @return the current marking.
	 */
	public String read ()
	{
		return markings [nthMark];
	}

	/**
	 * Resets the wheel to its first marking.
	 */
	public void reset ()
	{
		nthMark = 0;
	}

	/**
	 * Clones this object.
	 */
	public Object clone ()
	{
		Wheel clone = new Wheel (this.markings);
		clone.nthMark = this.nthMark;

		return clone;
	}
}
